@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2021 the xine project
+ * Copyright (C) 2000-2022 the xine project
  *
  * This file is part of xine, a free video player.
  *
@@ -1006,6 +1006,17 @@ int  xine_get_pos_length (xine_stream_t *stream,
  */
 uint32_t    xine_get_stream_info (xine_stream_t *stream, int info) XINE_PROTECTED;
 const char *xine_get_meta_info   (xine_stream_t *stream, int info) XINE_PROTECTED;
+#define XINE_QUERY_STREAM_INFO 1
+/** @brief query multiple stream props, thread safe and consistently.
+ *  @param stream  the xine stream to query.
+ *  @param sbuf    the buffer to write string copies to. sbuf[0] will be set to 0.
+ *  @param sblen   the byte size of sbuf.
+ *  @param strings input: -1 terminated list of XINE_META_INFO_*.
+ *                 output: list of offsets into sbuf or 0 for NULL strings.
+ *  @param ints    input: -1 terminated list of XINE_STREAM_INFO_*.
+ *                 output: list of values or 0 where not set or found.
+ *  @return        the count of bytes used in sbuf. */
+int xine_query_stream_info (xine_stream_t *stream, char *sbuf, size_t sblen, int *strings, int *ints) XINE_PROTECTED;
 
 /* xine_get_stream_info */
 #define XINE_STREAM_INFO_BITRATE           0
